import os
import time
import numpy as np
import richdem as rd
from scipy import ndimage
from skimage import measure
from osgeo import gdal, ogr, osr


# class for true depression
class Depression:
    def __init__(self, id, count, size, volume, meanDepth, maxDepth, minElev, bndElev, perimeter, major_axis,
                 minor_axis, elongatedness, eccentricity, orientation, area_bbox_ratio):
        self.id = id
        self.count = count
        self.size = size
        self.volume = volume
        self.meanDepth = meanDepth
        self.maxDepth = maxDepth
        self.minElev = minElev
        self.bndElev = bndElev
        self.perimeter = perimeter
        self.major_axis = major_axis
        self.minor_axis = minor_axis
        self.elongatedness = elongatedness
        self.eccentricity = eccentricity
        self.orientation = orientation
        self.area_bbox_ratio = area_bbox_ratio


# identify regions based on region growing method
def regionGroup(img_array, min_size, no_data):
    img_array[img_array == no_data] = 0
    label_objects, nb_labels = ndimage.label(img_array)
    sizes = np.bincount(label_objects.ravel())
    mask_sizes = sizes > min_size
    mask_sizes[0] = 0
    image_cleaned = mask_sizes[label_objects]
    label_objects, nb_labels = ndimage.label(image_cleaned)
    # nb_labels is the total number of objects. 0 represents background object.
    return label_objects, nb_labels


# convert numpy array to rdarray
def np2rdarray(in_array, no_data, projection, geotransform):
    out_array = rd.rdarray(in_array, no_data=no_data)
    out_array.projection = projection
    out_array.geotransform = geotransform
    return out_array


# compute depression attributes
def get_dep_props(objects, resolution):
    dep_list = []

    for object in objects:
        unique_id = object.label
        count = object.area
        size = count * pow(resolution, 2)  # depression size
        min_elev = np.float(object.min_intensity)  # depression min elevation
        max_elev = np.float(object.max_intensity)  # depression max elevation
        max_depth = max_elev - min_elev  # depression max depth
        mean_depth = np.float((max_elev * count - np.sum(object.intensity_image)) / count)  # depression mean depth
        volume = mean_depth * count * pow(resolution, 2)  # depression volume
        perimeter = object.perimeter * resolution
        major_axis = object.major_axis_length * resolution
        minor_axis = object.minor_axis_length * resolution
        if minor_axis == 0:
            minor_axis = resolution
        elongatedness = major_axis * 1.0 / minor_axis
        eccentricity = object.eccentricity
        orientation = object.orientation / 3.1415 * 180
        area_bbox_ratio = object.extent

        dep_list.append(Depression(unique_id, count, size, volume, mean_depth, max_depth, min_elev, max_elev, perimeter,
                                   major_axis, minor_axis, elongatedness, eccentricity, orientation, area_bbox_ratio))

    return dep_list


# save the depression list info to csv
def write_dep_csv(dep_list, csv_file):
    csv = open(csv_file, "w")
    header = "region-id" + "," + "count"+"," + "area" + "," + "volume" + "," + "avg-depth" + "," + "max-depth" + \
             "," + "min-elev" + "," + "max-elev" + "," + "perimeter" + "," + "major-axis" + "," + "minor-axis" + \
             "," + "elongatedness" + "," + "eccentricity" + "," + "orientation" + "," + "area-bbox-ratio"

    csv.write(header + "\n")
    for dep in dep_list:
        line = "{},{},{:.2f},{:.2f},{:.2f},{:.2f},{:.2f},{:.2f}, {:.2f},{:.2f},{:.2f},{:.2f},{:.2f},{:.2f},{:.2f}".\
            format(dep.id, dep.count, dep.size, dep.volume,dep.meanDepth, dep.maxDepth, dep.minElev, dep.bndElev,
            dep.perimeter, dep.major_axis, dep.minor_axis, dep.elongatedness, dep.eccentricity, dep.orientation,
            dep.area_bbox_ratio)
        csv.write(line + "\n")
    csv.close()


# raster to vector
def polygonize(img,shp_path):
    # mapping between gdal type and ogr field type
    type_mapping = {gdal.GDT_Byte: ogr.OFTInteger,
                    gdal.GDT_UInt16: ogr.OFTInteger,
                    gdal.GDT_Int16: ogr.OFTInteger,
                    gdal.GDT_UInt32: ogr.OFTInteger,
                    gdal.GDT_Int32: ogr.OFTInteger,
                    gdal.GDT_Float32: ogr.OFTReal,
                    gdal.GDT_Float64: ogr.OFTReal,
                    gdal.GDT_CInt16: ogr.OFTInteger,
                    gdal.GDT_CInt32: ogr.OFTInteger,
                    gdal.GDT_CFloat32: ogr.OFTReal,
                    gdal.GDT_CFloat64: ogr.OFTReal}

    ds = gdal.Open(img)
    prj = ds.GetProjection()
    srcband = ds.GetRasterBand(1)

    dst_layername = "Shape"
    drv = ogr.GetDriverByName("ESRI Shapefile")
    dst_ds = drv.CreateDataSource(shp_path)
    srs = osr.SpatialReference(wkt=prj)

    dst_layer = dst_ds.CreateLayer(dst_layername, srs=srs)
    # raster_field = ogr.FieldDefn('id', type_mapping[srcband.DataType])
    raster_field = ogr.FieldDefn('id', type_mapping[gdal.GDT_Int32])
    dst_layer.CreateField(raster_field)
    gdal.Polygonize(srcband, srcband, dst_layer, 0, [], callback=None)
    del img, ds, srcband, dst_ds, dst_layer


# extract sinks from dem
def ExtractSinks(in_dem, min_size, out_dir):

    start_time = time.time()

    out_dem = os.path.join(out_dir, "dem.tif")
    out_dem_filled = os.path.join(out_dir, "dem_filled.tif")
    out_dem_diff = os.path.join(out_dir, "dem_diff.tif")
    out_sink = os.path.join(out_dir, "sink.tif")
    out_region = os.path.join(out_dir, "region.tif")
    out_depth = os.path.join(out_dir, "depth.tif")
    out_csv_file = os.path.join(out_dir, "regions_info.csv")
    out_vec_file = os.path.join(out_dir, "regions.shp")

    # create output folder if nonexistent
    if not os.path.exists(out_dir):
        os.mkdir(out_dir)

    # load the dem and get dem info
    print("Loading data ...")
    dem = rd.LoadGDAL(in_dem)
    no_data = dem.no_data
    projection = dem.projection
    geotransform = dem.geotransform
    cell_size = np.round(geotransform[1], decimals=2)

    # get min and max elevation of the dem
    max_elev = np.float(np.max(dem))
    min_elev = np.float(np.min(dem[dem > 0]))
    print("min = {:.2f}, max = {:.2f}, no_data = {}, cell_size = {}".format(min_elev, max_elev, no_data, cell_size))

    # depression filling
    print("Depression filling ...")
    dem_filled = rd.FillDepressions(dem, in_place=False)
    dem_diff = dem_filled - dem
    dem_diff.no_data = 0

    print("Saving filled dem ...")
    rd.SaveGDAL(out_dem_filled, dem_filled)
    rd.SaveGDAL(out_dem_diff, dem_diff)

    # nb_labels is the total number of objects. 0 represents background object.
    print("Region grouping ...")
    label_objects, nb_labels = regionGroup(dem_diff, min_size, no_data)
    dem_diff[label_objects == 0] = 0
    depth = np2rdarray(dem_diff, no_data=0, projection=projection, geotransform=geotransform)
    rd.SaveGDAL(out_depth, depth)
    del dem_diff, depth

    print("Computing properties ...")
    objects = measure.regionprops(label_objects, dem, coordinates='xy')
    dep_list = get_dep_props(objects, cell_size)
    write_dep_csv(dep_list, out_csv_file)
    del objects, dep_list

    # convert numpy to richdem data format
    region = np2rdarray(label_objects, no_data=0, projection=projection, geotransform=geotransform)
    del label_objects

    print("Saving sink dem ...")
    sink = np.copy(dem)
    sink[region == 0] = 0
    sink = np2rdarray(sink, no_data=0, projection=projection, geotransform=geotransform)
    rd.SaveGDAL(out_sink, sink)
    # del sink

    print("Saving refined dem ...")
    dem_refined = dem_filled
    dem_refined[region > 0] = dem[region > 0]
    dem_refined = np2rdarray(dem_refined, no_data=no_data, projection=projection, geotransform=geotransform)
    rd.SaveGDAL(out_dem, dem_refined)
    rd.SaveGDAL(out_region, region)
    del dem_refined, region, dem

    print("Converting raster to vector ...")
    polygonize(out_region, out_vec_file)

    end_time = time.time()
    print("Total run time:\t\t\t {:.4f} s\n".format(end_time - start_time))

    return out_sink


if __name__ == '__main__':

    # ************************ change the following parameters if needed ******************************** #
    # set input files
    in_dem = "data/dem.tif"
    # parameters for depression filling
    min_size = 1000        # minimum number of pixels as a depression
    min_depth = 0.3        # minimum depression depth
    # set output directory
    out_dir = os.path.join(os.path.expanduser("~"), "temp")  # create a temp folder under user home directory
    # ************************************************************************************************** #

    sink_path = ExtractSinks(in_dem, min_size=min_size, out_dir=out_dir)
    dem = rd.LoadGDAL(in_dem)
    sink = rd.LoadGDAL(sink_path)
    demfig = rd.rdShow(dem, ignore_colours=[0], axes=False, cmap='jet', figsize=(6, 5.5))
    sinkfig = rd.rdShow(sink, ignore_colours=[0], axes=False, cmap='jet', figsize=(6, 5.5))

    print("Results are saved in: {}".format(out_dir))




import os
import pkg_resources
import richdem as rd
from .filtering import MedianFilter, MeanFilter, GaussianFilter
from .filling import ExtractSinks
from .slicing import DelineateDepressions
from .mounts import DelineateMounts
# from lidar import *
import PySimpleGUI as sg

def main():
    # identify the sample data directory of the package
    package_name = 'lidar'
    data_dir = pkg_resources.resource_filename(package_name, 'data/')

    # use the sample dem. Change it to your own dem if needed
    in_dem = os.path.join(data_dir, 'dem.tif')
    # set output directory. By default, use the temp directory under user's home directory
    out_dir = os.path.join(os.path.expanduser("~"), "temp")

    if not os.path.exists(out_dir):
        os.mkdir(out_dir)

    with sg.FlexForm('lidar package GUI') as form:
        form_rows = [
            [sg.Text('Level-set Method for Delineating Topographic Hierarchy', size=(50, 1), font=("Arial", 14), text_color='black')],
            [sg.Text('Select DEM:', font=("Arial", 14))],
            [sg.InputText(in_dem, size=(60, 1)), sg.FileBrowse()],
            [sg.Text('Delineation Mode:', font=("Arial", 14))],
            [sg.Radio('Depressions', "RADIO1", default=True), sg.Radio('Mounts', "RADIO1")],
            [sg.Text('DEM Filtering:', font=("Arial", 14))],
            [sg.Text('Select Filter:'), sg.InputCombo(['None', 'Mean Filter', 'Median Filter', 'Gaussian Filter']), sg.Text('Kernel Size: '), sg.InputText(default_text='3', size=(10, 1))],
            [sg.Text('Level-set Parameters:', font=("Arial", 14))],
            [sg.Text('Minimum size:'), sg.InputText(default_text='1000', size=(10, 1)), sg.Text('Minimum depth:'), sg.InputText(default_text='1.0', size=(10, 1))],
            [sg.Text('Slicing interval:'), sg.InputText(default_text='0.5', size=(10, 1)), sg.Text('Output shapefiles:'), sg.InputCombo(['Yes', 'No'], default_value='No')],
            [sg.Text('Display Results:', font=("Arial", 14))],
            [sg.InputCombo(['Yes', 'No'], default_value='No')],
            [sg.Text('Select Output Directory:', font=("Arial", 14))],
                    [sg.InputText(out_dir, size=(60, 1)), sg.FolderBrowse()],
                    [sg.Submit(), sg.Cancel()]]
        button, (in_dem, mode_dep, mode_mnt, filter_type, kernel_szie, min_size, min_depth, interval, bool_shp, display, out_dir) = form.LayoutAndRead(form_rows)
        # sg.Popup(button, source_filename)

        if button == 'Submit':

            kernel_szie = int(kernel_szie)
            min_size = int(min_size)
            min_depth = float(min_depth)
            interval = float(interval)
            if bool_shp == 'Yes':
                bool_shp = True
            else:
                bool_shp = False
            if display == 'Yes':
                display = True
            else:
                display = False
            if mode_mnt and in_dem == os.path.join(data_dir, 'dem.tif'):
                in_dem = os.path.join(data_dir, 'dsm.tif')

            out_dem_name = filter_type.split(" ")[0].lower() + '.tif'
            out_dem = os.path.join(out_dir, out_dem_name)

            sg.Popup('Please Wait!', 'The program is running! You will receive another message when it is done!')

            if filter_type == 'Mean Filter':                
                in_dem = MeanFilter(in_dem, kernel_size=kernel_szie, out_file=out_dem)
            elif filter_type == 'Median Filter':
                in_dem = MedianFilter(in_dem, kernel_size=kernel_szie, out_file=out_dem)
            elif filter_type == 'Gaussian Filter':
                in_dem = GaussianFilter(in_dem, sigma=kernel_szie, out_file=out_dem)

            if mode_dep:
                sink_path = ExtractSinks(in_dem, min_size, out_dir)
                dep_id_path, dep_level_path = DelineateDepressions(sink_path, min_size, min_depth, interval, out_dir, bool_shp)
            else:
                sink_path = os.path.join(out_dir, 'sink.tif')
                dep_id_path, dep_level_path = DelineateMounts(in_dem, min_size, min_depth, interval, out_dir, bool_shp)

            if display:
                # loading data and results
                dem = rd.LoadGDAL(in_dem)
                sink = rd.LoadGDAL(sink_path)
                dep_id = rd.LoadGDAL(dep_id_path)
                dep_level = rd.LoadGDAL(dep_level_path)

                # plotting results
                dem_fig = rd.rdShow(dem, ignore_colours=[0], axes=False, cmap='jet', figsize=(6, 5.5))
                sink_fig = rd.rdShow(sink, ignore_colours=[0], axes=False, cmap='jet', figsize=(6, 5.5))
                dep_id_fig = rd.rdShow(dep_id, ignore_colours=[0], axes=False, cmap='jet', figsize=(6, 5.5))
                dep_level_path = rd.rdShow(dep_level, ignore_colours=[0], axes=False, cmap='jet', figsize=(6, 5.5))

                del dem, sink, dep_id, dep_level, dem_fig, sink_fig, dep_id_fig, dep_id_path

            sg.Popup('Success!','The results are saved in: {}'.format(out_dir))

def GUI():
    main()

def gui():
    main()

if __name__ == '__main__':
    main()


import os
import pkg_resources
import richdem as rd
from scipy import ndimage
import numpy as np
import time

# convert numpy array to rdarray
def np2rdarray(in_array, no_data, projection, geotransform):
    out_array = rd.rdarray(in_array, no_data=no_data)
    out_array.projection = projection
    out_array.geotransform = geotransform
    return out_array


# mean filter
def MeanFilter(in_dem, kernel_size=3, out_file=None):

    print("Mean filtering ...")
    start_time = time.time()
    dem = rd.LoadGDAL(in_dem)
    no_data = dem.no_data
    projection = dem.projection
    geotransform = dem.geotransform

    weights = np.full((kernel_size, kernel_size), 1.0/(kernel_size * kernel_size))
    mean = ndimage.filters.convolve(dem, weights)
    mean = np2rdarray(mean, no_data, projection, geotransform)
    print("Run time: {:.4f} seconds".format(time.time() - start_time))

    if out_file is not None:
        print("Saving dem ...")
        rd.SaveGDAL(out_file, mean)
        return out_file

    return mean


# median filter
def MedianFilter(in_dem, kernel_size=3, out_file=None):

    print("Median filtering ...")
    start_time = time.time()
    dem = rd.LoadGDAL(in_dem)
    no_data = dem.no_data
    projection = dem.projection
    geotransform = dem.geotransform

    med = ndimage.median_filter(dem, size=kernel_size)
    med = np2rdarray(med, no_data, projection, geotransform)
    print("Run time: {:.4f} seconds".format(time.time() - start_time))

    if out_file is not None:
        print("Saving dem ...")
        rd.SaveGDAL(out_file, med)
        return out_file

    return med


# Gaussian filter
def GaussianFilter(in_dem, sigma=1, out_file=None):

    print("Gaussian filtering ...")
    start_time = time.time()
    dem = rd.LoadGDAL(in_dem)
    no_data = dem.no_data
    projection = dem.projection
    geotransform = dem.geotransform

    gau = ndimage.gaussian_filter(dem, sigma=sigma)
    gau = np2rdarray(gau, no_data, projection, geotransform)
    print("Run time: {:.4f} seconds".format(time.time() - start_time))

    if out_file is not None:
        print("Saving dem ...")
        rd.SaveGDAL(out_file, gau)
        return out_file

    return gau


# #####################################  main script
if __name__ == '__main__':

    # identify the sample data directory of the package
    package_name = 'lidar'
    data_dir = pkg_resources.resource_filename(package_name, 'data/')
    print("Sample data directory: {}".format(data_dir))

    # use the sample dem. Change it to your own dem if needed
    in_dem = os.path.join(data_dir, 'dem.tif')
    out_dir = os.path.join(os.path.expanduser("~"), "temp")

    if not os.path.exists(out_dir):
        os.mkdir(out_dir)

    median_dem = os.path.join(out_dir, "median.tif")
    gaussian_dem = os.path.join(out_dir, "gaussian.tif")
    mean_dem = os.path.join(out_dir, "mean.tif")

    dem = rd.LoadGDAL(in_dem)                   # original dem

    mean = MeanFilter(in_dem, kernel_size=3)
    mean_diff = mean - dem
    med = MedianFilter(in_dem, kernel_size=3)
    med_diff = med - dem
    gau = GaussianFilter(in_dem, sigma=1)
    gau_diff = gau - dem

    # plotting data
    dem_fig = rd.rdShow(dem, ignore_colours=[0], axes=False, cmap='jet', figsize=(6, 5.5))
    mean_fig = rd.rdShow(mean, ignore_colours=[0], axes=False, cmap='jet', figsize=(6, 5.5))
    med_fig = rd.rdShow(med, ignore_colours=[0], axes=False, cmap='jet', figsize=(6, 5.5))
    gau_fig = rd.rdShow(gau, ignore_colours=[0], axes=False, cmap='jet', figsize=(6, 5.5))
    mean_diff_fig = rd.rdShow(mean_diff, ignore_colours=[0], axes=False, cmap='jet', figsize=(6, 5.5))
    med_diff_fig = rd.rdShow(med_diff, ignore_colours=[0], axes=False, cmap='jet', figsize=(6, 5.5))
    gau_diff_fig = rd.rdShow(gau_diff, ignore_colours=[0], axes=False, cmap='jet', figsize=(6, 5.5))

    # save results
    rd.SaveGDAL(mean_dem, mean)
    rd.SaveGDAL(median_dem, med)
    rd.SaveGDAL(gaussian_dem, gau)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""The setup script."""
import os, platform
from os import path as op
import io
from setuptools import setup, find_packages

# check GDAL version installed in the system
# GDAL_VERSION = os.popen("gdal-config --version").read().rstrip()
GDAL_INFO = os.popen("gdalinfo --version").read().rstrip()
GDAL_VERSION = GDAL_INFO.split(',')[0].replace('GDAL', '').lstrip()
GDAL_VERSION_NUM = str(GDAL_VERSION.replace(".", ""))
PYGDAL_VERSION = '2.2.2.3' # default pygdal version to install


# pygdal version to install based on the GDAL version
# GDAL version history: https://trac.osgeo.org/gdal/wiki/DownloadSource
# pygdal version history: https://pypi.org/project/pygdal/#history
if not GDAL_VERSION_NUM.isdigit():
    print("GDAL cannot be detected in your system. Please install GDAL first!")
elif GDAL_VERSION[:3] == '2.3':
    PYGDAL_VERSION = GDAL_VERSION + '.4'
else:
    PYGDAL_VERSION = GDAL_VERSION + '.3'

with open('README.rst', mode = 'rb') as readme_file:
    readme = readme_file.read().decode('utf-8')

with open('HISTORY.rst', mode = 'rb') as history_file:
    history = history_file.read().decode('utf-8')

here = op.abspath(op.dirname(__file__))

# get the dependencies and installs
with io.open(op.join(here, 'requirements.txt'), encoding='utf-8') as f:
    all_reqs = f.read().split('\n')

install_requires = [x.strip() for x in all_reqs if 'git+' not in x]
if platform.system() != "Windows":
    install_requires.append('pygdal==' + PYGDAL_VERSION)
dependency_links = [x.strip().replace('git+', '') for x in all_reqs if 'git+' not in x]

requirements = ['Click>=6.0', ]

setup_requirements = [ ]

test_requirements = [ ]

setup(
    author="Qiusheng Wu",
    author_email='giswqs@gmail.com',
    classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Natural Language :: English',
        "Programming Language :: Python :: 2",
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',

    ],
    description="Terrain and hydrological analysis based on LiDAR-derived digital elevation models (DEM)",
    entry_points={
        'console_scripts': [
            'lidar=lidar.cli:main',
        ],
    },
    install_requires=install_requires,
    dependency_links=dependency_links,
    license="MIT license",
    long_description=readme + '\n\n' + history,
    include_package_data=True,
    keywords='lidar',
    name='lidar',
    packages=find_packages(include=['lidar']),
    setup_requires=setup_requirements,
    test_suite='tests',
    tests_require=test_requirements,
    url='https://github.com/giswqs/lidar',
    version='0.2.1',
    zip_safe=False,
)



import os
import pkg_resources
import richdem as rd
from filtering import MedianFilter
from filling import ExtractSinks
from slicing import DelineateDepressions

# identify the sample data directory of the package
package_name = 'lidar'
data_dir = pkg_resources.resource_filename(package_name, 'data/')

# use the sample dem. Change it to your own dem if needed
in_dem = os.path.join(data_dir, 'dem.tif')
# set output directory. By default, use the temp directory under user's home directory
out_dir = os.path.join(os.path.expanduser("~"), "temp")

if not os.path.exists(out_dir):
    os.mkdir(out_dir)

# parameters for identifying sinks and delineating nested depressions
min_size = 1000         # minimum number of pixels as a depression
min_depth = 0.3         # minimum depth as a depression
interval = 0.3          # slicing interval for the level-set method
bool_shp = False        # output shapefiles for each individual level

# extracting sinks based on user-defined minimum depression size
out_dem = os.path.join(out_dir, "median.tif")
in_dem = MedianFilter(in_dem, kernel_size=3, out_file=out_dem)
sink_path = ExtractSinks(in_dem, min_size, out_dir)
dep_id_path, dep_level_path = DelineateDepressions(sink_path, min_size, min_depth, interval, out_dir, bool_shp)

print('Results are saved in: {}'.format(out_dir))

# loading data and results
dem = rd.LoadGDAL(in_dem)
sink = rd.LoadGDAL(sink_path)
dep_id = rd.LoadGDAL(dep_id_path)
dep_level = rd.LoadGDAL(dep_level_path)

# plotting results
dem_fig = rd.rdShow(dem, ignore_colours=[0], axes=False, cmap='jet', figsize=(6, 5.5))
sink_fig = rd.rdShow(sink, ignore_colours=[0], axes=False, cmap='jet', figsize=(6, 5.5))
dep_id_fig = rd.rdShow(dep_id, ignore_colours=[0], axes=False, cmap='jet', figsize=(6, 5.5))
dep_level_path = rd.rdShow(dep_level, ignore_colours=[0], axes=False, cmap='jet', figsize=(6, 5.5))

